/*
 * JDBReport Generator
 * 
 * Copyright (C) 2006-2008 Andrey Kholmanskih. All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, write to the 
 *
 * Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330,
 * Boston, MA  USA  02111-1307
 * 
 * Andrey Kholmanskih
 * support@jdbreport.com
 * 
 */
package jdbreport.grid;

import java.awt.GridBagLayout;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.List;

import jdbreport.model.Cell;

/**
 * @version 1.1 03/09/08
 * @author Andrey Kholmanskih
 * 
 */
public class CellPropertiesPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JCheckBox noPrintBox = null;

	private JCheckBox noEditBox = null;
	private List<Cell> cells;

	/**
	 * This is the default constructor
	 */
	public CellPropertiesPanel() {
		super();
		initialize();
	}

	public void setCells(List<Cell> cells) {
		this.cells = cells;
		if (cells != null)
			initValues();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.anchor = GridBagConstraints.NORTHWEST;
		gridBagConstraints1.weighty = 0.1;
		gridBagConstraints1.weightx = 0.1;
		gridBagConstraints1.insets = new Insets(4, 16, 4, 4);
		gridBagConstraints1.gridy = 1;
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.insets = new Insets(16, 16, 4, 4);
		gridBagConstraints.weightx = 0.1;
		gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
		gridBagConstraints.gridy = 0;
		this.setSize(300, 200);
		this.setLayout(new GridBagLayout());
		this.add(getNoPrintBox(), gridBagConstraints);
		this.add(getNoEditBox(), gridBagConstraints1);
	}

	private void initValues() {
		noPrintBox.setSelected(cells.get(0).isNotPrint());
		noEditBox.setSelected(!cells.get(0).isEditable());
	}

	/**
	 * This method initializes noPrintBox
	 * 
	 * @return javax.swing.JCheckBox
	 */
	private JCheckBox getNoPrintBox() {
		if (noPrintBox == null) {
			noPrintBox = new JCheckBox();
			noPrintBox.setText(Messages
					.getString("CellPropertiesPanel.notprint")); //$NON-NLS-1$
		}
		return noPrintBox;
	}

	/**
	 * This method initializes noEditBox
	 * 
	 * @return javax.swing.JCheckBox
	 */
	private JCheckBox getNoEditBox() {
		if (noEditBox == null) {
			noEditBox = new JCheckBox();
			noEditBox
					.setText(Messages.getString("CellPropertiesPanel.notedit")); //$NON-NLS-1$
		}
		return noEditBox;
	}

	public void apply() {
		for (Cell cell : cells) {
			cell.setNotPrint(noPrintBox.isSelected());
			cell.setEditable(!noEditBox.isSelected());
		}
	}
	
}
